//#define __DEBUG__
#include <iomanip>
#ifndef __CINT__
#include <sstream>
#include "TROOT.h"
#include "TSystem.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH2.h"
#include "TTree.h"

#include "EXTPCKalDetector.h"
#include "kaltest/TKalDetCradle.h"

#include "JREvent.h"
#include "JCluster.h"
#include "JHitMaker.h"
#include "JHitCorrector.h"
#include "JTrackMaker.h"
#include "JSimpleTrackFitter.h"
#include "JTripletTrackFinder.h"
#include "JTrack.h"
#endif

auto kChargeMinCut = 0.;

void hitmon(Int_t run = 19987)
{
#if 0
  gInterpreter->AddIncludePath("../../../KalTest/v02-03/include");
  gInterpreter->AddIncludePath("../../include");
#endif

  // ---------------
  // Open input file
  // ---------------
  stringstream finstr;
  finstr << "../unpacker/root/readout-" << run << "_0.root" << ends;
  cerr << "Infile: " << finstr.str().data() << endl;
  auto   fp = new TFile(finstr.str().data());
  auto  &f  = *fp;
  auto   tp = static_cast<TTree *>(f.Get("Event"));
  JREvent *eventp = nullptr;
  tp->SetBranchAddress("Event", &eventp);
  auto nentries = static_cast<Int_t>(tp->GetEntries());
  cerr << "nentries = " << nentries << endl;

  // ----------------
  // Open output file
  // ----------------
  stringstream dirname;
  dirname << "HitData" << ends;
  if (gSystem->AccessPathName(dirname.str().data())){
    gSystem->mkdir(dirname.str().data(),kTRUE);
    cerr << "Create new directory: " << dirname.str().data() << endl;
  }
  stringstream foutstr;
  foutstr << dirname.str().data() << "/run" << run << ".root" << ends;
  cerr << "Outfile: " << foutstr.str().data() << endl;
  auto hfp = new TFile(foutstr.str().data(),"RECREATE","Marilin");

  // --------------------
  // Create Reco. classes
  // --------------------

  JHitMaker        hitmaker;
  TKalDetCradle    lp1;
  EXTPCKalDetector tpcdet;
  lp1.Install(tpcdet);
  //lp1.SwitchOffMS();

  // --------------------
  // Now loop over events
  // --------------------
  const Int_t kMaxEvt = 20000;
  //const Int_t kMaxEvt = 2000;

  std::map<std::string, TH2D *> hitmap;
  auto hqavp = new TH2D("hQav","",20000,0.,20000.,300,0.,3000);

  for (auto ev=0; ev<nentries && ev<kMaxEvt; ev++) {
      tp->GetEntry(ev);
      if (!(ev%100)) {
        cerr << "ev: " << ev << " evno: " << eventp->GetEventNumber() << " #cls: " << eventp->GetEntries() << endl;
      }
      //cerr << " -------<Event " << ev << " >----------------" << endl;
      JCluster::SetSortBy(JCluster::kDet); // set sorting policy
      eventp->Sort(); // sort clusters in ascending order of layers
      hitmaker.MakeHits(eventp);
      if (!(ev%100)) {
        //hitmaker.DebugPrint("detail");
        hitmaker.DebugPrint();
      }
      TIter next(&hitmaker);
      JHitLayer *hlp;
      auto nhits  = 0;
      auto chgsum = 0.;
      while ((hlp = static_cast<JHitLayer *>(next()))) {
          TIter nexthit(hlp);
	  JHit *hp = nullptr;
          while ((hp = static_cast<JHit *>(nexthit()))) { // loop ver hits
	      auto xv     = hp->GetPosition();
	      auto module = hp->GetModuleID();
	      auto row    = hp->GetLayerID();
	      auto charge = hp->GetCharge();
	      if (charge < kChargeMinCut) continue;
	      chgsum += charge;
	      nhits++;
	      stringstream hname;
	      hname << "hCharge" << module << "_" << row << ends; // histogram name
	      if (!hitmap[hname.str()]) {
                  hitmap[hname.str()] = new TH2D(hname.str().data(),"",20000,0.,20000.,1000,0.,1000);
	      }
	      auto hpp = hitmap[hname.str()];
	      hpp->Fill(ev+0.1, charge);
	  }
      }
      if (nhits) hqavp->Fill(ev+0.1, chgsum/nhits);
  } 
  // -----------------------
  // Now output tree headers
  // -----------------------

  hqavp->Write();
  for (const auto& [key, hpp]: hitmap) {
     if (hpp) hpp->Write();
  }

  hfp->Write();
}
