//#define __DEBUG__
#include <iomanip>
#ifndef __CINT__
#include <sstream>
#include "TROOT.h"
#include "TSystem.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH2.h"
#include "TTree.h"

#include "EXTPCKalDetector.h"
#include "kaltest/TKalDetCradle.h"

#include "JREvent.h"
#include "JCluster.h"
#include "JHitMaker.h"
#include "JHitCorrector.h"
#include "JTrackMaker.h"
#include "JSimpleTrackFitter.h"
#include "JTripletTrackFinder.h"
#include "JTrack.h"
#endif
//-----------------------------------//
//  Pulse shape (height/width) cuts  //
//-----------------------------------//
///////////////////////////////////////
const double kCosThetaCut = 0.98;
const int    kMaxMissing  =  20;
const int    kMinNhits    =  10;
const int    kZdiffCut    = 5.0;
///////////////////////////////////////

void trackhitmon(Int_t run = 19987, const Char_t *delxfile = "Results/deltax0.dat")
{
#if 0
  gInterpreter->AddIncludePath("../../../KalTest/v02-03/include");
  gInterpreter->AddIncludePath("../../include");
#endif

  // ---------------
  // Open input file
  // ---------------
  stringstream finstr;
  finstr << "../unpacker/root/readout-" << run << "_0.root" << ends;
  cerr << "Infile: " << finstr.str().data() << endl;
  auto  fp = new TFile(finstr.str().data());
  auto &f  = *fp;
  auto  T  = static_cast<TTree *>(f.Get("Event"));
  JREvent *event = nullptr;
  T->SetBranchAddress("Event", &event);
  auto nentries = static_cast<Int_t>(T->GetEntries());
  cerr << "nentries = " << nentries << endl;

  // ----------------
  // Open output file
  // ----------------
  stringstream dirname;
  dirname << "HitData" << ends;
  if (gSystem->AccessPathName(dirname.str().data())){
    gSystem->mkdir(dirname.str().data(),kTRUE);
    cerr << "Create new directory: " << dirname.str().data() << endl;
  }
  stringstream foutstr;
  foutstr << dirname.str().data() << "/run" << run << ".root" << ends;
  cerr << "Outfile: " << foutstr.str().data() << endl;
  auto hfp = new TFile(foutstr.str().data(),"RECREATE","Marilin");

  // --------------------
  // Create Reco. classes
  // --------------------
  cerr << "distortion correction file = " << delxfile << endl;
  (void) JHitCorrector::GetInstancePtr(delxfile);
  JHitMaker        hitmaker;
  TKalDetCradle    lp1;
  EXTPCKalDetector tpcdet;
  lp1.Install(tpcdet);
  //lp1.SwitchOffMS();

  JSimpleTrackFitter fitter(&tpcdet);
  JTrack::SetTrackFitter(&fitter);

  JTrackMaker trackmaker;
  trackmaker.SetTrackFinder(new JTripletTrackFinder(&trackmaker));
  JTripletTrackFinder::SetMinNhits   (kMinNhits);
  JTripletTrackFinder::SetMaxMissing (kMaxMissing);
  JTripletTrackFinder::SetCosThetaCut(kCosThetaCut);
  JTripletTrackFinder::SetZdiffCut(kZdiffCut);
#if 1
#if 0
  trackmaker.KillModule(1);
  trackmaker.KillModule(6);
#else
  trackmaker.KillModule(0);
  //trackmaker.KillModule(3);
  trackmaker.KillModule(5);
#if 0
  // Exclude row 44 from the fit
  Int_t layertolookat = 16;
  JSimpleTrackFitter::SetIsOutFlag();
  JSimpleTrackFitter::SetLayerToLookAt(layertolookat);
#endif
#endif
#endif
  // --------------------
  // Now loop over events
  // --------------------
  const Int_t kMaxEvt = 20000;

  std::map<std::string, TH2D *> hitmap;
  auto hqavp = new TH2D("hQav","",20000,0.,20000.,300,0.,3000);

  for (Int_t ev=0; ev<nentries && ev<kMaxEvt; ev++) {
      T->GetEntry(ev);
      if (!(ev%100)) {
        cerr << "ev: " << ev << " evno: " << event->GetEventNumber() << " #cls: " << event->GetEntries() << endl;
      }
      cerr << " -------<Event " << ev << " >----------------" << endl;
      JCluster::SetSortBy(JCluster::kDet); // set sorting policy
      event->Sort(); // sort clusters in ascending order of layers
      hitmaker.MakeHits(event);
      //hitmaker.DebugPrint("detail");
      trackmaker.MakeTracks(&hitmaker);
      trackmaker.DebugPrint("detail");
#if 1
      auto ntrks = trackmaker.GetEntries();
      if (ntrks != 1) continue;
#endif
      TIter next(&trackmaker);
      JTrack *tp = nullptr;
      auto nhits  = 0;
      auto chgsum = 0.;
      while ((tp = static_cast<JTrack *>(next()))) {      // loop over tracks
          TIter nexthit(tp);
	  JHit *hp = nullptr;
          while ((hp = static_cast<JHit *>(nexthit()))) { // loop ver hits
	      auto xv     = hp->GetPosition();
	      auto module = hp->GetModuleID();
	      auto row    = hp->GetLayerID();
	      auto charge = hp->GetCharge();
	      chgsum += charge;
	      nhits++;
	      stringstream hname;
	      hname << "hCharge" << module << "_" << row << ends; // histogram name
	      if (!hitmap[hname.str()]) {
                  hitmap[hname.str()] = new TH2D(hname.str().data(),"",20000,0.,20000.,1000,0.,1000);
	      }
	      auto hpp = hitmap[hname.str()];
	      hpp->Fill(ev+0.1, charge);
	  }
      }
      if (nhits) hqavp->Fill(ev+0.1, chgsum/nhits);
  } 
  // -----------------------
  // Now output tree headers
  // -----------------------
  auto hTracks = static_cast<TNtupleD *>(gROOT->FindObject("hTracks"));
  auto hResXin = static_cast<TNtupleD *>(gROOT->FindObject("hResXin"));
  auto hResZin = static_cast<TNtupleD *>(gROOT->FindObject("hResZin"));

  hfp->cd();
  hTracks->Write();
  hResXin->Write();
  hResZin->Write();

  hqavp->Write();

  for (const auto& [key, hpp]: hitmap) {
       if (hpp) hpp->Write();
  }

  hfp->Write();
}
